@@ -20,15 +20,7 @@ import numpy as np
 
 import raven.struct.dataframe.core as dataframe
 import raven.struct.dataframe.column as column
-import raven.struct.dataframe.bytecolumn as bytecolumn
-import raven.struct.dataframe.shortcolumn as shortcolumn
-import raven.struct.dataframe.intcolumn as intcolumn
-import raven.struct.dataframe.longcolumn as longcolumn
-import raven.struct.dataframe.floatcolumn as floatcolumn
-import raven.struct.dataframe.doublecolumn as doublecolumn
-import raven.struct.dataframe.stringcolumn as stringcolumn
-import raven.struct.dataframe.charcolumn as charcolumn
-import raven.struct.dataframe.binarycolumn as binarycolumn
+import raven.struct.dataframe._columnutils as utils
 
 class BooleanColumn(column.Column):
     """A Column holding boolean values (bool).
@@ -139,25 +131,25 @@ class BooleanColumn(column.Column):
     # pylint: disable=invalid-name
     def convert_to(self, typecode):
         converted = None
-        if typecode == bytecolumn.ByteColumn.TYPE_CODE:
-            converted = bytecolumn.ByteColumn(values=self._values.astype(np.int8))
-        elif typecode == shortcolumn.ShortColumn.TYPE_CODE:
-            converted = shortcolumn.ShortColumn(values=self._values.astype(np.int16))
-        elif typecode == intcolumn.IntColumn.TYPE_CODE:
-            converted = intcolumn.IntColumn(values=self._values.astype(np.int32))
-        elif typecode == longcolumn.LongColumn.TYPE_CODE:
-            converted = longcolumn.LongColumn(values=self._values.astype(np.int64))
-        elif typecode == stringcolumn.StringColumn.TYPE_CODE:
+        if typecode == utils.type_code_byte_column():
+            converted = dataframe.DataFrame.ByteColumn(values=self._values.astype(np.int8))
+        elif typecode == utils.type_code_short_column():
+            converted = dataframe.DataFrame.ShortColumn(values=self._values.astype(np.int16))
+        elif typecode == utils.type_code_int_column():
+            converted = dataframe.DataFrame.IntColumn(values=self._values.astype(np.int32))
+        elif typecode == utils.type_code_long_column():
+            converted = dataframe.DataFrame.LongColumn(values=self._values.astype(np.int64))
+        elif typecode == utils.type_code_string_column():
             vals = self._values.astype(np.object)
             for i, x in np.ndenumerate(vals):
                 vals[i] = str(x)
 
-            converted = stringcolumn.StringColumn(values=vals)
-        elif typecode == floatcolumn.FloatColumn.TYPE_CODE:
-            converted = floatcolumn.FloatColumn(values=self._values.astype(np.float32))
-        elif typecode == doublecolumn.DoubleColumn.TYPE_CODE:
-            converted = doublecolumn.DoubleColumn(values=self._values.astype(np.float64))
-        elif typecode == charcolumn.CharColumn.TYPE_CODE:
+            converted = dataframe.DataFrame.StringColumn(values=vals)
+        elif typecode == utils.type_code_float_column():
+            converted = dataframe.DataFrame.FloatColumn(values=self._values.astype(np.float32))
+        elif typecode == utils.type_code_double_column():
+            converted = dataframe.DataFrame.DoubleColumn(values=self._values.astype(np.float64))
+        elif typecode == utils.type_code_char_column():
             vals = self._values.astype(np.uint8)
             for i, x in np.ndenumerate(vals):
                 if x:
@@ -165,42 +157,44 @@ class BooleanColumn(column.Column):
                 else:
                     vals[i] = ord("0")
 
-            converted = charcolumn.CharColumn(values=vals)
+            converted = dataframe.DataFrame.CharColumn(values=vals)
         elif typecode == BooleanColumn.TYPE_CODE:
             converted = self.clone()
-        elif typecode == binarycolumn.BinaryColumn.TYPE_CODE:
+        elif typecode == utils.type_code_binary_column():
             vals = np.empty([self._values.shape[0]], dtype=np.object)
             for i, x in np.ndenumerate(self._values):
                 vals[i] = bytearray(int(x).to_bytes(1, byteorder="big", signed=True))
 
-            converted = binarycolumn.BinaryColumn(values=vals)
-        elif typecode == bytecolumn.NullableByteColumn.TYPE_CODE:
+            converted = dataframe.DataFrame.BinaryColumn(values=vals)
+        elif typecode == utils.type_code_nullable_byte_column():
             vals = self._values.astype(np.int8)
-            converted = bytecolumn.NullableByteColumn(values=vals.astype(np.object))
-        elif typecode == shortcolumn.NullableShortColumn.TYPE_CODE:
+            converted = dataframe.DataFrame.NullableByteColumn(values=vals.astype(np.object))
+        elif typecode == utils.type_code_nullable_short_column():
             vals = self._values.astype(np.int16)
-            converted = shortcolumn.NullableShortColumn(values=vals.astype(np.object))
-        elif typecode == intcolumn.NullableIntColumn.TYPE_CODE:
+            converted = dataframe.DataFrame.NullableShortColumn(values=vals.astype(np.object))
+        elif typecode == utils.type_code_nullable_int_column():
             vals = self._values.astype(np.int32)
-            converted = intcolumn.NullableIntColumn(values=self._values.astype(np.object))
-        elif typecode == longcolumn.NullableLongColumn.TYPE_CODE:
+            converted = dataframe.DataFrame.NullableIntColumn(values=self._values.astype(np.object))
+        elif typecode == utils.type_code_nullable_long_column():
             vals = self._values.astype(np.int64)
-            converted = longcolumn.NullableLongColumn(values=self._values.astype(np.object))
-        elif typecode == stringcolumn.NullableStringColumn.TYPE_CODE:
+            converted = dataframe.DataFrame.NullableLongColumn(
+                values=self._values.astype(np.object))
+
+        elif typecode == utils.type_code_nullable_string_column():
             vals = self._values.astype(np.object)
             for i, x in np.ndenumerate(vals):
                 vals[i] = str(x)
 
-            converted = stringcolumn.NullableStringColumn(values=vals)
-        elif typecode == floatcolumn.NullableFloatColumn.TYPE_CODE:
+            converted = dataframe.DataFrame.NullableStringColumn(values=vals)
+        elif typecode == utils.type_code_nullable_float_column():
             vals = self._values.astype(np.float32)
             vals = vals.astype(np.object)
-            converted = floatcolumn.NullableFloatColumn(values=vals)
-        elif typecode == doublecolumn.NullableDoubleColumn.TYPE_CODE:
+            converted = dataframe.DataFrame.NullableFloatColumn(values=vals)
+        elif typecode == utils.type_code_nullable_double_column():
             vals = self._values.astype(np.float64)
             vals = vals.astype(np.object)
-            converted = doublecolumn.NullableDoubleColumn(values=vals)
-        elif typecode == charcolumn.NullableCharColumn.TYPE_CODE:
+            converted = dataframe.DataFrame.NullableDoubleColumn(values=vals)
+        elif typecode == utils.type_code_nullable_char_column():
             vals = self._values.astype(np.uint8)
             vals = vals.astype(np.object)
             for i, x in np.ndenumerate(vals):
@@ -209,15 +203,15 @@ class BooleanColumn(column.Column):
                 else:
                     vals[i] = ord("0")
 
-            converted = charcolumn.NullableCharColumn(values=vals)
+            converted = dataframe.DataFrame.NullableCharColumn(values=vals)
         elif typecode == NullableBooleanColumn.TYPE_CODE:
             converted = NullableBooleanColumn(values=self._values.astype(np.object))
-        elif typecode == binarycolumn.NullableBinaryColumn.TYPE_CODE:
+        elif typecode == utils.type_code_nullable_binary_column():
             vals = np.empty([self._values.shape[0]], dtype=np.object)
             for i, x in np.ndenumerate(self._values):
                 vals[i] = bytearray(int(x).to_bytes(1, byteorder="big", signed=True))
 
-            converted = binarycolumn.NullableBinaryColumn(values=vals)
+            converted = dataframe.DataFrame.NullableBinaryColumn(values=vals)
         else:
             raise dataframe.DataFrameException(
                 "Unknown column type code: {}".format(typecode))
@@ -340,60 +334,60 @@ class NullableBooleanColumn(column.Column):
     # pylint: disable=invalid-name
     def convert_to(self, typecode):
         converted = None
-        if typecode == bytecolumn.ByteColumn.TYPE_CODE:
+        if typecode == utils.type_code_byte_column():
             vals = np.empty([self._values.shape[0]], dtype=np.int8)
             for i, x in np.ndenumerate(self._values):
                 vals[i] = int(x) if x is not None else 0
 
-            converted = bytecolumn.ByteColumn(values=vals)
-        elif typecode == shortcolumn.ShortColumn.TYPE_CODE:
+            converted = dataframe.DataFrame.ByteColumn(values=vals)
+        elif typecode == utils.type_code_short_column():
             vals = np.empty([self._values.shape[0]], dtype=np.int16)
             for i, x in np.ndenumerate(self._values):
                 vals[i] = int(x) if x is not None else 0
 
-            converted = shortcolumn.ShortColumn(values=vals)
-        elif typecode == intcolumn.IntColumn.TYPE_CODE:
+            converted = dataframe.DataFrame.ShortColumn(values=vals)
+        elif typecode == utils.type_code_int_column():
             vals = np.empty([self._values.shape[0]], dtype=np.int32)
             for i, x in np.ndenumerate(self._values):
                 vals[i] = int(x) if x is not None else 0
 
-            converted = intcolumn.IntColumn(values=vals)
-        elif typecode == longcolumn.LongColumn.TYPE_CODE:
+            converted = dataframe.DataFrame.IntColumn(values=vals)
+        elif typecode == utils.type_code_long_column():
             vals = np.empty([self._values.shape[0]], dtype=np.int64)
             for i, x in np.ndenumerate(self._values):
                 vals[i] = int(x) if x is not None else 0
 
-            converted = longcolumn.LongColumn(values=vals)
-        elif typecode == stringcolumn.StringColumn.TYPE_CODE:
-            vals = np.empty([self._values.shape[0]], dtype=np.bool)
+            converted = dataframe.DataFrame.LongColumn(values=vals)
+        elif typecode == utils.type_code_string_column():
+            vals = np.empty([self._values.shape[0]], dtype=np.object)
             for i, x in np.ndenumerate(self._values):
                 if x is not None:
                     vals[i] = str(x)
                 else:
-                    vals[i] = stringcolumn.StringColumn.DEFAULT_VALUE
+                    vals[i] = utils.default_value_string_column()
 
-            converted = stringcolumn.StringColumn(values=vals)
-        elif typecode == floatcolumn.FloatColumn.TYPE_CODE:
+            converted = dataframe.DataFrame.StringColumn(values=vals)
+        elif typecode == utils.type_code_float_column():
             vals = np.empty([self._values.shape[0]], dtype=np.float32)
             for i, x in np.ndenumerate(self._values):
                 vals[i] = float(x) if x is not None else 0.0
 
-            converted = floatcolumn.FloatColumn(values=vals)
-        elif typecode == doublecolumn.DoubleColumn.TYPE_CODE:
+            converted = dataframe.DataFrame.FloatColumn(values=vals)
+        elif typecode == utils.type_code_double_column():
             vals = np.empty([self._values.shape[0]], dtype=np.float64)
             for i, x in np.ndenumerate(self._values):
                 vals[i] = float(x) if x is not None else 0.0
 
-            converted = doublecolumn.DoubleColumn(values=vals)
-        elif typecode == charcolumn.CharColumn.TYPE_CODE:
-            vals = self._values.astype(np.uint8)
-            for i, x in np.ndenumerate(vals):
+            converted = dataframe.DataFrame.DoubleColumn(values=vals)
+        elif typecode == utils.type_code_char_column():
+            vals = np.zeros([self._values.shape[0]], dtype=np.uint8)
+            for i, x in np.ndenumerate(self._values):
                 if x is not None and x is True:
                     vals[i] = ord("1")
                 else:
                     vals[i] = ord("0")
 
-            converted = charcolumn.CharColumn(values=vals)
+            converted = dataframe.DataFrame.CharColumn(values=vals)
         elif typecode == BooleanColumn.TYPE_CODE:
             vals = self._values.astype(np.bool)
             for i, x in np.ndenumerate(vals):
@@ -403,7 +397,7 @@ class NullableBooleanColumn(column.Column):
                     vals[i] = False
 
             converted = BooleanColumn(values=vals)
-        elif typecode == binarycolumn.BinaryColumn.TYPE_CODE:
+        elif typecode == utils.type_code_binary_column():
             vals = np.empty([self._values.shape[0]], dtype=np.object)
             for i, x in np.ndenumerate(self._values):
                 if x is not None:
@@ -411,50 +405,50 @@ class NullableBooleanColumn(column.Column):
                 else:
                     vals[i] = bytearray(int(0).to_bytes(1, byteorder="big", signed=True))
 
-            converted = binarycolumn.BinaryColumn(values=vals)
-        elif typecode == bytecolumn.NullableByteColumn.TYPE_CODE:
+            converted = dataframe.DataFrame.BinaryColumn(values=vals)
+        elif typecode == utils.type_code_nullable_byte_column():
             vals = np.empty([self._values.shape[0]], dtype=np.object)
             for i, x in np.ndenumerate(self._values):
                 vals[i] = int(x) if x is not None else None
 
-            converted = bytecolumn.NullableByteColumn(values=vals)
-        elif typecode == shortcolumn.NullableShortColumn.TYPE_CODE:
+            converted = dataframe.DataFrame.NullableByteColumn(values=vals)
+        elif typecode == utils.type_code_nullable_short_column():
             vals = np.empty([self._values.shape[0]], dtype=np.object)
             for i, x in np.ndenumerate(self._values):
                 vals[i] = int(x) if x is not None else None
 
-            converted = shortcolumn.NullableShortColumn(values=vals)
-        elif typecode == intcolumn.NullableIntColumn.TYPE_CODE:
+            converted = dataframe.DataFrame.NullableShortColumn(values=vals)
+        elif typecode == utils.type_code_nullable_int_column():
             vals = np.empty([self._values.shape[0]], dtype=np.object)
             for i, x in np.ndenumerate(self._values):
                 vals[i] = int(x) if x is not None else None
 
-            converted = intcolumn.NullableIntColumn(values=vals)
-        elif typecode == longcolumn.NullableLongColumn.TYPE_CODE:
+            converted = dataframe.DataFrame.NullableIntColumn(values=vals)
+        elif typecode == utils.type_code_nullable_long_column():
             vals = np.empty([self._values.shape[0]], dtype=np.object)
             for i, x in np.ndenumerate(self._values):
                 vals[i] = int(x) if x is not None else None
 
-            converted = longcolumn.NullableLongColumn(values=vals)
-        elif typecode == stringcolumn.NullableStringColumn.TYPE_CODE:
+            converted = dataframe.DataFrame.NullableLongColumn(values=vals)
+        elif typecode == utils.type_code_nullable_string_column():
             vals = self._values.astype(np.object)
             for i, x in np.ndenumerate(vals):
                 vals[i] = str(x) if x is not None else None
 
-            converted = stringcolumn.NullableStringColumn(values=vals)
-        elif typecode == floatcolumn.NullableFloatColumn.TYPE_CODE:
+            converted = dataframe.DataFrame.NullableStringColumn(values=vals)
+        elif typecode == utils.type_code_nullable_float_column():
             vals = np.empty([self._values.shape[0]], dtype=np.object)
             for i, x in np.ndenumerate(self._values):
                 vals[i] = float(x) if x is not None else None
 
-            converted = floatcolumn.NullableFloatColumn(values=vals)
-        elif typecode == doublecolumn.NullableDoubleColumn.TYPE_CODE:
+            converted = dataframe.DataFrame.NullableFloatColumn(values=vals)
+        elif typecode == utils.type_code_nullable_double_column():
             vals = np.empty([self._values.shape[0]], dtype=np.object)
             for i, x in np.ndenumerate(self._values):
                 vals[i] = float(x) if x is not None else None
 
-            converted = doublecolumn.NullableDoubleColumn(values=vals)
-        elif typecode == charcolumn.NullableCharColumn.TYPE_CODE:
+            converted = dataframe.DataFrame.NullableDoubleColumn(values=vals)
+        elif typecode == utils.type_code_nullable_char_column():
             vals = self._values.astype(np.object)
             vals = vals.astype(np.object)
             for i, x in np.ndenumerate(vals):
@@ -466,10 +460,10 @@ class NullableBooleanColumn(column.Column):
                 else:
                     vals[i] = None
 
-            converted = charcolumn.NullableCharColumn(values=vals)
+            converted = dataframe.DataFrame.NullableCharColumn(values=vals)
         elif typecode == NullableBooleanColumn.TYPE_CODE:
             converted = self.clone()
-        elif typecode == binarycolumn.NullableBinaryColumn.TYPE_CODE:
+        elif typecode == utils.type_code_nullable_binary_column():
             vals = np.empty([self._values.shape[0]], dtype=np.object)
             for i, x in np.ndenumerate(self._values):
                 if x is not None:
@@ -477,7 +471,7 @@ class NullableBooleanColumn(column.Column):
                 else:
                     vals[i] = None
 
-            converted = binarycolumn.NullableBinaryColumn(values=vals)
+            converted = dataframe.DataFrame.NullableBinaryColumn(values=vals)
         else:
             raise dataframe.DataFrameException(
                 "Unknown column type code: {}".format(typecode))
